/* $Revision$ $Date$ $Author$ */
/***********************************************************************
* ADSTAR Distributed Storage Manager (adsm)                            *
* Server Component                                                     *
*                                                                      *
* IBM Confidential                                                     *
* (IBM Confidential-Restricted when combined with the Aggregated OCO   *
* Source Modules for this Program)                                     *
*                                                                      *
* OCO Source Materials                                                 *
*                                                                      *
* 5648-020 (C) Copyright IBM Corporation 1990, 1997 (Unpublished)      *
***********************************************************************/

/***********************************************************************
* Name:
*        JSTree.js
*
* Function:
*        Provide a Dynamic tree in a web browser.
*
* Author:
*        BJ Scheid
*        Roy Tritch
*
* Status:
*        Version 3, Release 2
*
* Environment:
*        This is a PLATFORM-INDEPENDENT source file. As such it may
*        contain no dependencies on any specific operating system
*        environment or hardware platform.
*
* Public Routines:
*
* Description:
*        ...
*
* Design Notes:
*        ...
*
* Change activity:
*        ...
*
*  $Log$
*
***********************************************************************/

// Golbal Variables
var tree_root = null;
var maxNodes = 0;
var browser = 0;
var nodeIndex = null;
var totalWidth = 0;

/////////////////////////////////////////////////////////////
// Define a tree node
//
// Inputs: 
//   icon - name of the icon eg. /file/btn.gif
//   content - node or foler content (can be HTML text)
//   link - link for the name and icon
//   parent - parent node
//   id - id assigned to the node
/////////////////////////////////////////////////////////////
function TreeNode(icon, content, link, parent, target, id, level)
{
  // Class Variables
  this.browser = 0;		// Browser Type
  this.icon = icon;             // Content Icon
  this.link = link;             // Content Link
  this.level = level;           // Nodes Level wrt root node
  this.content = content;       // Content to be displayed
  this.parent = parent;         // Parent TreeNode
  this.target = target;         // Content Target (_blank for new window)
  this.children = new Array;    // Children of the current TreeNode
  this.childCount = 0;          // Number of children of this TreeNode
  this.id = id;                 // TreeNode Id number
  this.layer = null;            // Layer refering to the content displayed
  this.width = 0;               // Width of the layer
  this.height = 0;              // Height of the layer
  this.expanded = false;        // Are the children visiable
  this.visible = false;         // Is the node itself visiable
  this.PSrc = null;             // Plus icon source url
  this.MSrc = null;             // Minus icon source url
  this.button = null;           // Browser image object to the PM button

  // Class Methods 
  this.displayTree = displayTree;
  this.displayFlatTree = displayFlatTree;
  this.layout = JSTree_Layout;  

}
/////////////////////////////////////////////////////////////
// initalize()
//   Display the inital state of the tree.
//
// Inputs: 
//   None.
// Output:
//   Tree is displayed in the browser.
// Notes:
//   The tree has already been built by the setTreeRoot and
//   addTreeNode methods.
/////////////////////////////////////////////////////////////
function initalize()
{
  var node = tree_root;

  // setup the node index for faster access to tree members
  nodeIndex = new Array();
  createNodeIndex(node);
  
  // set the document node to the tree root
  document.node = tree_root;
  
  // In Netscape create a layer that is big enough for all
  // nodes to be displayed.  This is to force the scrollbars
  // to appear since Netscape is not as dynamic as IE when it
  // comes to scrollbars
  
  if(node.browser == 1) {
    var k;
    // Find the maximum width of the tree...
    for(k = 0; k < nodeIndex.length; k++)
    {
      var ndWidth = (nodeIndex[k].content.length * 7) + (22 * nodeIndex[k].level);
      if(ndWidth > totalWidth)
	totalWidth = ndWidth;
    }

    // The total height is the number node multiplied by the
    // number of icons plus some rubber chicken number.
    var totalHeight = (nodeIndex.length * 22) + 20;

   
    document.write("<LAYER ID=Sizer VISIBILITY=SHOW WIDTH="+totalWidth+" HEIGHT="+totalHeight+" Z-INDEX=0></LAYER>");
  }

  // display the tree
  // display the tree
  if( browser == 0 )
  {
    displayFlatTree( node, "" );
  }
  else
  {
    displayTree(node, "", false, true);

    // Expose the root 
    node.visible = true;
    if(browser == 1)
      node.layer.visibility="SHOW";
    else
      document.all["Item0"].style.display="block";

    // Show root's childrens
    node.expanded = true;
    expandChildren(node);
  
    // layout all the created layer in Netscape
    if(node.browser == 1)
      node.layout();
  }
}

/////////////////////////////////////////////////////////////
// createNodeIndex()
//   Create an index of all the nodes in the tree to speedup
//   lookup work.
//
// Inputs: 
//   The root of the tree.
// Output:
//   The nodeIndex is created.
/////////////////////////////////////////////////////////////
function createNodeIndex(node)
{
  nodeIndex[node.id] = node;

  var i;
  for(i = 0; i < node.childCount; i++)
  {
    createNodeIndex(node.children[i]);
  }
}

/////////////////////////////////////////////////////////////
// displayTree()
//   display the tree built in memory
//
// Inputs:
//   node - the node to show
//   spacer - html text spacer
//   lastNode - boolean flag inidcating last child
//   root - is this node the root node.
// Outputs:
//   The tree is displayed to the browser's window
/////////////////////////////////////////////////////////////
function displayTree(node, spacer, lastnode, root)
{ 
  // build the item layer
  if(node.browser == 1)
  {
    document.write("<LAYER ID=Item"+node.id+" VISIBILITY=HIDE WIDTH="+totalWidth+" Z-INDEX=1>");
  } 
  else
  {
    document.write("<DIV ID=Item"+node.id+"  style={display:none} NOWRAP>");
  }

  if(!root)
  {
    //add the spacer to the item
    document.write(spacer);
  }
  
  // Set the node's layer
  if(node.browser == 1)
    node.layer = document.layers['Item'+node.id];
  else
    node.layer = document.all["Item"+node.id];

  // Determine what or if plus-minus buttons are needed.
  if(node.childCount >0)
  {
    if(!root)
    {
      // add the appropate  buttons
      if(lastnode)
      {
	node.PSrc = "ASHHX911.GIF";
	node.MSrc = "ASHHX910.GIF";
        document.write("<A HREF='javascript:expandCompressNode("+this.id+")'>");
	document.write("<IMG SRC='"+node.PSrc+"' NAME='PM"+node.id+"' WIDTH=22"+
		       " HEIGHT=22 ALIGN=MIDDLE BORDER=0>");
	document.write("</A>");

	spacer += "<IMG SRC=ASHHX904.GIF WIDTH=22 ALIGN=MIDDLE HEIGHT=22 BORDER=0>";
      }
      else
      {
	node.PSrc = "ASHHX915.GIF";
	node.MSrc = "ASHHX914.GIF";
	document.write("<A HREF='javascript:expandCompressNode("+node.id+")'>");
	document.write("<IMG SRC='"+node.PSrc+"' NAME='PM"+node.id+"' WIDTH=22"+
		       " HEIGHT=22 ALIGN=MIDDLE BORDER=0>");
	document.write("</A>");
	spacer += "<IMG SRC=ASHHX916.GIF WIDTH=22 ALIGN=MIDDLE HEIGHT=22 BORDER=0>";
      }

      // set the node button to the appropate browser image object
      if(node.browser == 1)
	node.button = node.layer.document.images['PM'+node.id];
      else
	node.button = document.all['PM'+node.id];
    }

    //write the content of the node out
    document.write("<A HREF="+node.link+" TARGET="+node.target+">");
    document.write("<IMG SRC="+node.icon+" ALT='"+node.content+"' WIDTH=22 ALIGN=MIDDLE HEIGHT=22"+
		   " BORDER=0 >");
    document.write(""+node.content+"</A>");

    // close the item layer
    if(node.browser == 1)
      document.write("</LAYER>");
    else
      document.write("</DIV>"); 

  
    // loop around all the children and add them
    var i;
    for(i = 0; i < node.childCount; i++)
    {
      var lstnode = false;
      if(i == (node.childCount -1))
	lstnode = true;

      node.children[i].displayTree(node.children[i], spacer, lstnode, false);
    }

  } // end if( has children)
  else
  {
    // add node with out childern, note spacer does not change
    if(lastnode)
      document.write("<IMG SRC=ASHHX909.GIF WIDTH=22"+
		     " HEIGHT=22 ALIGN=MIDDLE BORDER=0>");
    else
      document.write("<IMG SRC=ASHHX913.GIF WIDTH=22"+
		     " HEIGHT=22 ALIGN=MIDDLE BORDER=0>");

    //write the content of the node out

    document.write("<A HREF="+node.link+" TARGET="+node.target+">");
    document.write("<IMG SRC="+node.icon+" ALT='"+node.content+"' WIDTH=22 HEIGHT=22"+
		   " BORDER=0 ALIGN=MIDDLE ALIGN=TOP>");
    document.write(""+node.content+"</A>");

    // close the item layer and find the longest content string
    if(node.browser == 1) 
      document.write("</LAYER>");
    else
      document.write("</DIV>"); 
  }
}

/////////////////////////////////////////////////////////////
// displayFlatTree()
//   Display the tree built in memory as a flat tree
//
// Inputs:
//   node - the node to show
//   spacer - html text spacer
//   lastNode - boolean flag inidcating last child
//   root - is this node the root node.
// Outputs:
//   The tree is displayed to the browser's window
/////////////////////////////////////////////////////////////
function displayFlatTree(node, spacer)
{ 
  //add the spacer to the item
  document.write(spacer);

  //write the content of the node out
  document.write("<A HREF="+node.link+" TARGET="+node.target+">");
  document.write("<IMG SRC="+node.icon+" ALT='"+node.content+"' WIDTH=22 ALIGN=MIDDLE"+
                   " HEIGHT=22 BORDER=0 >");
  document.write(""+node.content+"</A><BR>");

  // Determine what or if plus-minus buttons are needed.
  if(node.childCount > 0)
  {
 
    // loop around all the children and add them
    var i;
    for(i = 0; i < node.childCount; i++)
    {
      var newspacer = spacer + "&nbsp;&nbsp;";
      node.children[i].displayFlatTree( node.children[i], newspacer );
    }
  }
}

/////////////////////////////////////////////////////////////
// addItem()
//   Add an item to the tree
// Inputs: 
//   parent - parent node
//   icon - name of the icon eg. btn.gif
//   content - node or foler content (can be HTML text)
//   link - link for the name and icon
// Output:
//   The tree node that has been created.
/////////////////////////////////////////////////////////////
function addItem(parent, icon, content, link, target)
{
  if(tree_root == null)
    alert("ERROR:\n Adding nodes to a null tree.\n"+
	  "Define the root first with addRoot().");
  else if(parent == null)
    // assume that a null parent inferse adding to the root node
    parent = tree_root;

  var tempNode = new TreeNode(icon, content, link, parent, target, maxNodes++, parent.level + 1);
  tempNode.browser = browser;
 
  parent.children[parent.childCount++] = tempNode;

  return tempNode;
}

/////////////////////////////////////////////////////////////
// addRoot()
//   Set the root of the tree to be constructed
// Inputs: 
//   parent - parent node
//   icon - name of the icon eg. btn.gif
//   content - node or foler content (can be HTML text)
//   link - link for the name and icon
// Output:
//   The root tree node that has been created.
/////////////////////////////////////////////////////////////
function addRoot(icon, content, link, target)
{
  // Determine the browser type
  // Browser Types:
  // 0 - Not supported
  // 1 - Netscape Communicator
  // 2 - IE 4.0
  var browserVer = parseInt(navigator.appVersion);
  var browserNam = navigator.appName;
  if(browserNam == "Netscape" && browserVer < 5 )
    browser = 1;
  else if( browserNam.indexOf( "Microsoft" ) != -1 && browserVer >= 4 )
    browser = 2;
  else
  {
    browser = 0;
  }
  // create a new node and set that as the root.
  var tempNode = new TreeNode(icon, content, link, null, target, maxNodes++, 1);
  tempNode.browser = browser;

  tree_root = tempNode;
  return tempNode;
}

/////////////////////////////////////////////////////////////
// expandChildren()
//   Expand the chlidren of the node by making the chlidren
//   nodes visable.
//
// Inputs: 
//   node - the node to be expanded
// Output:
//   None.
/////////////////////////////////////////////////////////////
function expandChildren(node)
{
  var i;
  for(i = 0; i < node.childCount; i++)
  {
    var cnode = node.children[i]; 

    // show the layers of this node
    if(cnode.browser == 1)
      document.layers['Item'+cnode.id].visibility="show";
    else
      document.all["Item"+cnode.id].style.display = "block";

    cnode.visible = true;

    // open up any previouly open children    
    if(cnode.expanded)
      expandChildren(cnode);
  }
}

/////////////////////////////////////////////////////////////
// expandCompressNode()
//   Event Handler for the Plus-Minus buttons.  This procedure
//   will either expand a node or compress a node depending on
//   the current state of node.
//
// Inputs: 
//   id - the id number of the node of which created the event
// Output:
//   None.
/////////////////////////////////////////////////////////////
function expandCompressNode(id)
{
  var node = nodeIndex[id];

  // determine the current state of the node
  if(!node.expanded)
  {
    // Expand the node
    node.button.src = node.MSrc;
    node.expanded = true;

    // Show all the children
    expandChildren(node);
  }
  else
  {
    //Compress the node
    node.button.src = node.PSrc;
    node.expanded = false;

    // Hide all the Children
    compressChildren(node);
  }

  if(node.browser == 1)
    node.layout();
}
 
/////////////////////////////////////////////////////////////
// compressChildren()
//   Compress the chlidren of the node by hiding its chlidren
//   nodes.
//
// Inputs: 
//   node - the node to be compressed
// Output:
//   None.
///////////////////////////////////////////////////////////// 
function compressChildren(node)
{
  var i;
  for(i = 0; i < node.childCount; i++)
  {
    var cnode = node.children[i];

    // Handle the different browser cases
    if(cnode.browser == 1)
      document.layers['Item'+cnode.id].visibility="hide";
    else
      document.all["Item"+cnode.id].style.display = "none";

    // update the node visible flag
    cnode.visible = false;

    // close up the child's children
    if(cnode.childCount > 0)
      compressChildren(cnode);
  }
}

/////////////////////////////////////////////////////////////
// JSTree_Layout()
//   Layout the layers of the tree one after another.
//
// Inputs: 
//   None.  Uses document.layers to do the layout.
// Output:
//   A Laid out tree.
// Note:
//   This function is only needed for Netscape.
///////////////////////////////////////////////////////////// 
function JSTree_Layout()
{
  var docLayers;
  
  // extract the layers of the document
  docLayers = document.layers;

  // initalize position and loop variables
  var i = 0;
  var posY = docLayers[i].pageY + docLayers[i].document.height;

  for(i = 1; i < docLayers.length; i++)
  {
    // don't layout a hidden layer
    if(docLayers[i].visibility != "hide")
    {
      // set the location of the layer
      docLayers[i].moveTo(docLayers[i].x, posY);
      posY += docLayers[i].document.height;
    }
  }
}
